##################################################################################
# snackAmp - urlRedirect package 1.0 for snackAmp
# provides dynamic http applications for SnackAmp
# web server.
#
# Music File Player in Tcl/Tk
#
# Copyright (C) 2001 Tom Wilkason
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
#
# Please send any comments/bug reports to
# tom.wilkason@cox.net  (Tom Wilkason)
##################################################################################
#
# To create a new dynamic url create a namespace with the name of the url
# e.g. namespace eval playlist -> /playlist
# to enable it, create an init routine that returns a 1 (for security)
# you can have the url return either an existing file (e.g. an mp3 file), in
# that case create a namespace command ::process, that returns the name
# of the file.
#
# If you want to create dynamic html with the namespace, create a namespace
# command ::process, that returns content. You will need an extra parameter
# passed to it that you set to the file extension required.
#
set [file root [file tail [info script]]]_History {
   $Header: /cvsroot/snackamp/snackamp/lib/urlRedirect.tcl,v 1.9 2003/05/17 14:20:16 wilkason Exp $
}
##################################################################################
#                              j u s t c o n t r o l
##################################################################################

namespace eval remote/justcontrol {
   proc init { } {
     return 1
   }
}
proc remote/justcontrol::process {sock {params ""}} {
   Trace
   foreach param [split $params &]  {
      foreach {CmdType Cmd} [split $param =] {break}
      set Cmd [HttpD::Decode $Cmd]
      switch -- $CmdType {
         bt_Level {
            ;# Would be nice not to have to push data back
            ;# maybe hold to end to push it out if needed.
            if {[catch {::macros::bumpLevel $Cmd} result] } then {
            }
         }
         default {}
      } ;# End Switch
   } ;# end foreach
   return [list dhtml html 0]
}

##
# Wrapper for jukebox function, makes the browser namespace react based
# on calling function
#
##################################################################################
#                              j u k e b o x
##################################################################################
namespace eval jukebox/display {
   proc init { } {
     return 1
   }
}
###############################################################################
# Function   : display
# Description: Main function that is called by web server when /display
#              is entered.
#              e.g. /display
# Author     : Tom Wilkason
# Date       : 12/15/2002
###############################################################################
proc jukebox/display::process {sock {params ""}} {
   Trace
   append Result [remote/control::leader "SnackAmp Playlist" $::snackAmpSettings(cssName) ""]
   append Result "<body>\n"
   append Result ${::search::javascript}
   ;# Wait to let player take hold
   #append Result ${search::close_window}
   append Result [httpHelpers::sndInfo 0]
   append Result [remote/control::nextToPlay 2 $::snackAmpSettings(httpNextLength)]
   append Result "</body>\n</html>"
   return [list dhtml html $Result]
}

##
# Jukebox current playing
#
namespace eval jukebox/header {
   proc init { } {
     return 1
   }
   proc [namespace current]::process {sock {params ""}} {
      append Result [remote/control::leader "SnackAmp Playlist" $::snackAmpSettings(cssName)]
      append Result "<body>\n"
      append Result [httpHelpers::getRefreshScript 5 "/jukebox/header"]
      append result [H2 "Now [string tolower [soundControl::playState]]:"]
      append Result [httpHelpers::trackInfo [soundControl::fileName] 1]
      append Result "</body>\n</html>"
      return [list dhtml html $Result]
   }
}
##
# Jukebox next
#
namespace eval jukebox/next {
   proc init { } {
     return 1
   }
   proc [namespace current]::process {sock {params ""}} {
      append Result [remote/control::leader "SnackAmp Playlist" $::snackAmpSettings(cssName)]
      append Result "<body>\n"
      append Result [httpHelpers::getRefreshScript $::snackAmpSettings(refreshRate) "/jukebox/next"]
      append Result [remote/control::nextToPlay 2 10]
      append Result "</body>\n</html>"
      return [list dhtml html $Result]
   }
}
##
# Jukebox next
#
namespace eval jukebox/last {
   proc init { } {
     return 1
   }
   proc [namespace current]::process {sock {params ""}} {
      append Result [remote/control::leader "SnackAmp Playlist" $::snackAmpSettings(cssName)]
      append Result "<body>\n"
      append Result [httpHelpers::getRefreshScript $::snackAmpSettings(refreshRate) "/jukebox/last"]
      append Result [remote/control::queuedToPlay 2 20]
      append Result "</body>\n</html>"
      return [list dhtml html $Result]
   }
}
##
# Jukebox next
#
namespace eval jukebox/folder {
   proc init { } {
     return 1
   }
   proc [namespace current]::process {sock {params ""}} {
      set control 2
      append result [remote/control::leader "SnackAmp Playlist" $::snackAmpSettings(cssName)]
      append result "<body>\n"
      if {[string length $params]} {
         foreach param [split $params &]  {
            foreach {CmdType Cmd} [split $param =] {break}
            set dCmd [HttpD::Decode $Cmd]
            switch -- $CmdType {
               folder {
                  append result [H2 "Select a folder and pick your favorite track"]
                  append result [browser::guts $Cmd $control $params]
               }
            } ;# End Switch
         }
      } else {
         append result [H2 "Select a folder and pick your favorite track"]
         append result [browser::guts [info hostname] $control $params]
      }
      append result "</body>\n</html>\n"
      return [list dhtml html [string trim $result]]
   }
}
namespace eval jukebox/browser {
   proc init { } {
     return 1
   }
   proc process {sock {params ""}} {
      dirBrowser::doit $sock $params
   }
}
namespace eval jukebox/search {
   proc init { } {
     return 1
   }
   proc process {sock {params ""}} {
      search::doit $sock $params
   }
}
##################################################################################
#                               c o n t r o l
##################################################################################
##
# control namespace to control the SnackAmp player from
# a web browser. When user enters /control ... in to the browser
# The process function of this namespace will be called passing any
# user parameters. To add a new command then create another namespace
# and provide an init and process function.
#
##
# Wrapper for remote/search function, makes the search namespace react based
# on calling function
#
namespace eval remote/search {
   proc init { } {
     return 1
   }
   proc process {sock {params ""}} {
      search::doit $sock $params
   }
}
namespace eval remote/browser {
   proc init { } {
     return 1
   }
   proc process {sock {params ""}} {
      dirBrowser::doit $sock $params
   }
}
namespace eval remote/control {
   variable Details
   variable id3Cache
     ;# <meta http-equiv="Content-Type" content="text/html; charset=windows-1252">
    #  <meta http-equiv="pragma"           content="no-cache">

   variable head {
<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html>
   <head>
      <meta http-equiv="Content-Language" content="en-us">
      <meta name="GENERATOR"              content="SnackAmp Web Server">
      <meta name="ProgId"                 content="SnackAmp.Document">
      <LINK REL=StyleSheet                HREF="$css" TYPE="text/css" MEDIA=screen>
      <title>$title</title>
   }
   variable songTemplate {
      <table class="detailsTable">
        <tr>
          <td width="10%" class="songTemplateLabel">Title</td>
          <td width="90%" colspan="5" class="songTitle"><b>$Details(Title)</b></td>
        </tr>
        <tr>
          <td class="songTemplateLabel" width="10%">Artist</td>
          <td class="songTemplateInfo"  width="90%" colspan="5">$Details(Artist)</td>
        </tr>
        <tr>
          <td class="songTemplateLabel" width="10%">Album</td>
          <td class="songTemplateInfo" width="90%" colspan="5">$Details(Album)</td>
        </tr>
        <tr>
          <td class="songTemplateLabel" width="10%">File</td>
          <td class="songTemplateInfo" width="90%" colspan="5">$Details(Name)</td>
        </tr>
        <tr>
          <td class="songTemplateLabel" width="10%">Folder</td>
          <td class="songTemplateInfo" width="90%" colspan="5">$Details(Folder)</td>
        </tr>
        <tr>
          <td width="9%"   class="songTemplateLabel">Size</td>
          <td width="30%"  class="songTemplateInfo">$Details(fSize)</td>
          <td width="13%"  class="songTemplateLabel">Genre</td>
          <td width="20%"  class="songTemplateInfo">$Details(Genre)</td>
          <td width="10%"  class="songTemplateLabel">Gain</td>
          <td width="100%" class="songTemplateInfo">[gainControl::levelText $Details(levelFactor)]</td>
        </tr>
        <tr>
          <td width="9%"   class="songTemplateLabel">Date</td>
          <td width="30%"  class="songTemplateInfo">$Details(fDate)</td>
          <td width="13%"  class="songTemplateLabel">Rate</td>
          <td width="20%"  class="songTemplateInfo">$Details(rate)</td>
          <td width="10%"  class="songTemplateLabel">Track</td>
          <td width="100%" class="songTemplateInfo">$Details(Track)</td>
        </tr>
        <tr>
          <td width="9%"   class="songTemplateLabel">Year</td>
          <td width="30%"  class="songTemplateInfo">$Details(Year)</td>
          <td width="13%"  class="songTemplateLabel">Duration</td>
          <td width="20%"  class="songTemplateInfo">$Details(duration)</td>
          <td width="10%"  class="songTemplateLabel">Position</td>
          <td width="100%" class="songTemplateInfo">$Details(Position)</td>
        </tr>
      </table>
   }
   variable songInfoTemplate {
      <table class="detailsTable">
        <tr>
          <td class="songTemplateLabel">Title</td>
          <td class="songTitle"><b>$Details(Title)</b></td>
        </tr>
        <tr>
          <td class="songTemplateLabel" width="10%">Artist</td>
          <td class="songTemplateInfo"  width="90%" colspan="5">$Details(Artist)</td>
        </tr>
        <tr>
          <td class="songTemplateLabel" width="10%">Album</td>
          <td class="songTemplateInfo" width="90%" colspan="5">$Details(Album)</td>
        </tr>
      </table>
   }
   proc init { } {
     return 1
   }
}
###############################################################################
# Function   : remote/control::process state machine
# Description: Main function that is called by web server when /remote/control
#              is entered. Inside is a loop that iterates over each parameter
#              on the POST/GET and handles each one.
#              e.g. /remote/control?a=2&b=c  <= two params a= and b=
# Author     : Tom Wilkason
# Date       : 4/24/2002
###############################################################################
proc remote/control::process {sock {params ""}} {
   Trace
   ;##
   ;# Replace puts with appending to string then doing as a batch if needed
   ;#
   append Result [remote/control::leader "SnackAmp Control" $::snackAmpSettings(cssName) "/remote/control?Status=Refresh+Data"]
   append Result "<body>\n"
   append Result ${::search::javascript}
   #append Result "$params"
   foreach param [split $params &]  {
      foreach {CmdType Cmd} [split $param =] {break}
      set Cmd [HttpD::Decode $Cmd]
      switch -- $CmdType {
         bt_Control {
            if {[catch {::macros::$Cmd} result] } then {
               append Result "<p>$::errorInfo</p>\n"
            }
            after 50 set ::remote/control::flag 1
            vwait ::remote/control::flag
         }
         bt_Level {
            if {[catch {::macros::bumpLevel $Cmd} result] } then {
               append Result "<p>$::errorInfo</p>\n"
            }
         }
         play {
            ;# Play a specific song
            ;# /remote/control?play=some+name.mp3
            if {[catch {playSpecificSong $Cmd} error] } then {
               saLog $result
               append Result $result
            }
            after 50 set ::remote/control::flag 1
            vwait ::remote/control::flag
         }
         delete {
            LB::delete $Cmd
            updateFileDisplay
            ;# Remove from playlist
         }
         cmb_APL {
            ;# Select an auto-playlist file
            ;# TFW: If a real + is in file name, this may fail
            macros::readAPLFile [file join $::HOME $Cmd.tafs] 1
         }
         cmb_Sort {
            ;# Sort/Shuffle the files
            switch -glob -- [string tolower $Cmd] {
               sort*tracks     {set ::snackAmpSettings(sortMode) 0}
               shuffle*tracks  {set ::snackAmpSettings(sortMode) 1}
               shuffle*sort    {set ::snackAmpSettings(sortMode) 2}
               shuffle*shuffle {set ::snackAmpSettings(sortMode) 3}
            } ;# End Switch
            filtering::sortMusic
         }
         filter_text {
            ;# stage 1
            set ::TA(filter) $Cmd
            ;# Note: These will happen in this order, search_replace can process
            ;# Button=Filter&filter_text=ab&filter_what=All+Files&search_replace=Replace
         }
         filter_what {
            ;# stage 2
            switch -glob -- [string tolower $Cmd] {
               "all*files" {set Search_What 1}
               "play*" {set Search_What playList}
               "auto*" {set Search_What 0}
               default {set Search_What 1}
            } ;# End Switch
         }
         search_replace {
            ;# stage 3, do the filtering
            if {![info exists Search_What]} {set Search_What 1}
            set Search_Replace [string tolower $Cmd]
            filtering::pushFilter
            macros::filterMusic $Search_Replace $Search_What
            ;# stage 3
         }
         cmb_PL {
            ;# Stage 1
            ;# Set the playlist file
            foreach ext {m3u taf} {
               set Playlist_File [file join $::HOME $Cmd.$ext]
               if {[file exists $Playlist_File]} {
                  break
               }
            } ;# end foreach
         }
         PL_search_replace {
            ;# Stage 2
            ;# And then apply it
            macros::ProcessPlaylistFile $Playlist_File [string tolower $Cmd] 0
         }
         default {}
      } ;# End Switch
   } ;# end foreach
   ;# Wait to let player take hold
   append Result [httpHelpers::sndInfo]
   append Result [nextToPlay 1 $::snackAmpSettings(httpNextLength)]
   append Result "</body>\n</html>"
   return [list dhtml html $Result]
}
###############################################################################
# Function   : leader
# Description: Page leader for dymanic page
# Author     : Tom Wilkason
# Date       : 4/24/2002
###############################################################################
proc remote/control::leader {title css {url "xx"}} {
   Trace
   variable head
   ;# title is used in subst
   append result [subst $head]
   if {![string equal $url "xx"]} {
     append result \n
     append result [subst {    <META HTTP-EQUIV="Refresh" CONTENT="$::snackAmpSettings(refreshRate);URL=$url">}]
     append result \n
   } ;# end if
   append result "  </head>\n"
}
###############################################################################
# Function   : nextToPlay
# Description: Print out the next N tracks in the playlist
#
# Author     : Tom Wilkason
###############################################################################
proc remote/control::nextToPlay {playButton {N 20}} {
   Trace
   variable id3Cache
   global LB
   set offset 2
   set result {}
   if {[catch {
      set top [expr {$LB::indexPlaying-$offset}]
      set end [LB::index end]
      set total [expr {$end - [LB::active]}]
      set newEnd [expr {$top+$N+$offset}]
      if {$newEnd > $end} {set newEnd $end} ;# end if
      if {$top < 0} {set top 0} ;# end if
      set current [file tail $::TA(currentMF)]
      append result [H2 "Coming up next (of [commify $end] tracks)"]
      append result "<table class='songPlaylist'>"
      append result [remote/control::trackList $playButton $top $newEnd]
      append result </table>
#      append result </div>
   } error]} {
      return $error
   } else {
      return $result
   }
}
proc remote/control::queuedToPlay {playButton {N 20}} {
   Trace
   variable id3Cache
   global LB
   set offset $N
   set result {}
   if {[catch {
      set top [expr {$LB::indexPlaying+1}]
      set end $LB::webInsert
      if {$top < 0} {set top 0} ;# end if
      if {$end-$top <= 0} {
         append result [H2 "No tracks queued"]
      } else {
         #set current [file tail $::TA(currentMF)]
         append result [H2 "User queued tracks"]
         append result "<table class='songPlaylist'>"
         append result [remote/control::trackList $playButton $top $end]
         append result </table>
      }
#      append result </div>
   } error]} {
      return $error
   } else {
      return $result
   }
}
###############################################################################
# Function   :
# Description:
# Author     : Tom Wilkason
# Date       : 7/2/2003
###############################################################################
proc remote/control::trackList {playButton top newEnd} {
if {1} {
#      set fHeader "Artist-Album"
      set fHeader "Artist"
} else {
      set fHeader "Folder"
}
      switch -- $playButton {
         0 {append result [httpHelpers::tableRow [list Num $fHeader "Track Name"] headerRow]}
         1 {append result [httpHelpers::tableRow [list Num ? X $fHeader "Track Name"] headerRow]}
         2 {append result [httpHelpers::tableRow [list $fHeader "Track Name"] headerRow]}
         default {}
      }
      set ctr 0
      for {set i $top} {$i < $newEnd} {incr i} {
         set track ""
         if {$i == $LB::indexPlaying} {
            set bgc activeRow
         } else {
#            set bgc $httpHelpers::row_styles([expr {($i/5)&0x1}])
            set bgc [httpHelpers::rowStyle ctr 1]
         }
         set song [LB::get $i]
         set file [LB::getFullName $i 0]

         #TFW: //cache ID3 info on file and use it
if {1} {
         if {[info exists id3Cache($file)]} {
            ;# Cam also check timestamps for afresh copy
            array set Info $id3Cache($file)
            set Info($file) [array get Info]
#            set folder "$Info(Artist) - $Info(Album)"
            set folder $Info(Artist)
            set track $Info(Title)
         } else {
            catch {unset Info}
            foreach key {Artist Album Title} {
               set Info($key) ""
            } ;# end foreach
            if {[id3Tag::id3Label $file Info]} {
               ;# Clear out this entry on hour from now
               after [expr {60*60000}] [list httpHelpers::clean remote/control::ideCache($file)]
            }
            ;# Insert the tag data if available
            if {[string length $Info(Title)]==0} {
               set Info(Title) [id3Tag::guessTitle $file]
            }
            if {[string length $Info(Artist)]==0} {
                set Info(Artist) [id3Tag::guessArtist $file]
            }
            ;# Save for next time
            set id3Cache($file) [array get Info]
            set track $Info(Title)
            set folder $Info(Artist)
         }
} else {
            set folder [httpHelpers::displayFolder $file]
            set track [ROOTTAIL $file]
}
         ;##
         ;# Show the appropriate row with appropriate controls
         ;#
         switch -- $playButton {
            0 {
               ;# No Control
               if {$folder==""} {
                  set flink [SP]
               } else {
   # Note: hyperlink for /display only
                  set flink $folder
               }
               append result [httpHelpers::tableRow [list $i $flink $track] $bgc]
            }
            1 {
               ;# Full Control
               if {$folder==""} {
                  set flink [SP]
               } else {
                  ;# Hyperlink only if remote controling (i.e. playButton= 1)
                  # /remote is current (need to fix to use remote/browser as current)
                  set flink [AREF "browser?folder=[ENCODE [file dirname $file]]" $folder "Browse This Folder"]
               }
               set track "[httpHelpers::inLinePlay $song][NOAMP $track]"
               set imInfo [httpHelpers::popupInfo $song]
               set imDel [AREF "?delete=$i" $httpHelpers::deleteimage "Remove Track From Playlist"]
               append result [httpHelpers::tableRow [list $i $imInfo $imDel $flink $track] $bgc]
            }
            2 {
               ;# Just hyperlink (jukebox)
               if {$folder==""} {
                  set flink [SP]
               } else {
                  # /julebox/browser is current
                  set flink [AREF "?folder=[ENCODE [file dirname $file]]" $folder "Browse This folder"]
               }
               #set imInfo [httpHelpers::popupInfo $song]
               append result [httpHelpers::tableRow [list $flink $track] $bgc]
            }
            default {}
         }
      } ;# end for
      return $result
}
###############################################################################
# Function   : ::remote/control::aplFiles
# Description: Callback from TML file to return a list of all auto-playlist files
# Author     : Tom Wilkason
# Date       : 4/28/2002
###############################################################################
proc remote/control::aplFiles {args} {
   Trace
   set result ""
   ;# Create option list of all auto-playlist files
   foreach {file} [catalog::getAutoPlaylists] {
      set choice [ROOTTAIL $file]
      if {[string equal $choice $::TA(AutoPlayListSelection)]} {
         append result "<option selected>$choice</option>\n"
      } else {
         append result "<option>$choice</option>\n"
      }
   }
   return [string trim $result]
}
###############################################################################
# Function   : ::remote/control::plFiles
# Description: Callback from TML file to return a list of all playlist files
# Author     : Tom Wilkason
# Date       : 4/28/2002
###############################################################################
proc remote/control::plFiles {args} {
   Trace
   set result ""
   ;# Create option list of all auto-playlist files
   foreach {file} [catalog::getPlaylists] {
      set choice [ROOTTAIL $file]
      if {[string equal $choice $::TA(PlayListSelection)]} {
         append result "<option selected>$choice</option>\n"
      } else {
         append result "<option>$choice</option>\n"
      }
   } ;# end foreach
   return [string trim $result]
}
###############################################################################
# Function   : ::remote/control::songTable
# Description: Return HTML for the song details
# Author     : Tom Wilkason
# Date       : 4/29/2002
###############################################################################
proc remote/control::songTable {_Details} {
   upvar $_Details Details
   variable songTemplate
   set result [subst $songTemplate]
   return $result
}
proc remote/control::songInfo {_Details} {
   upvar $_Details Details
   variable songInfoTemplate
   set result [subst $songInfoTemplate]
   return $result
}

##################################################################################
#                              coverart
##################################################################################
namespace eval coverart {
   proc init { } {
     return 1
   }
}
;# Returns a local file as dynamic content, used for cover art
;# For security only handles specific file names
proc coverart::process {sock {params ""}} {
   variable cache
   if {[string length $params]} {
      foreach param [split $params &]  {
         foreach {CmdType Cmd} [split $param =] {break}
         switch -- $CmdType {
            get {
               set folder [HttpD::Decode $Cmd]
               set img [coverart::covername $folder]
               if {[string length $img]} {
                  set ext [string trimleft [file extension $img] .]
                  if {[info exists cache($img)]} {
                     return [list dhtml $ext $cache($img)]
                  } else {
                     if {[catch {
                        set fid [open $img r]
                        fconfigure $fid -translation binary
                        set data [read $fid]
                        close $fid
                     } result] } then {
                        return [list dhtml html $result]
                     } else {
                        set cache($img) $data
                        ;# remote cache entry after 5 minutes
                        after [expr {5*60000}] [list httpHelpers::clean ::coverart::cache($img)]
                        return [list dhtml $ext $data]
                     }
                  }
               }
            }
         } ;# end switch
      } ;# end foreach param
   } ;# end if
   return [list dhtml html ""]
}
##
# Determine if a valid coverart file exists in this folder
#
proc coverart::covername {folder} {
   foreach {image} $::snackAmpSettings(coverart) {
      set img [file join $folder $image]
      if {[file exists $img]} {
         return $img
      }
   }
   return ""
}

##################################################################################
#                              d i s p l a y
##################################################################################
namespace eval display {
   proc init { } {
     return 1
   }
}
###############################################################################
# Function   : display
# Description: Main function that is called by web server when /display
#              is entered.
#              e.g. /display
# Author     : Tom Wilkason
# Date       : 12/15/2002
###############################################################################
proc display::process {sock {params ""}} {
   Trace
   append Result [remote/control::leader "SnackAmp Playlist" $::snackAmpSettings(cssName) ""]
   append Result "<body>\n"
   append Result ${::search::javascript}
   ;# Wait to let player take hold
   #append Result ${search::close_window}
   append Result [httpHelpers::sndInfo 0]
   append Result [remote/control::nextToPlay 0 $::snackAmpSettings(httpNextLength)]
   append Result "</body>\n</html>"
   return [list dhtml html $Result]
}
##################################################################################
#                              s t r e a m i n g  f i l e  b r o w s e r
##################################################################################
namespace eval dirBrowser {
}
proc dirBrowser::leader {title css {url "xx"}} {
   Trace
   ;# title is used in subst
   append result [subst ${remote/control::head}]
   if {![string equal $url "xx"]} {
     append result \n
     append result [subst {    <META HTTP-EQUIV="Refresh" CONTENT="$::snackAmpSettings(refreshRate);URL=$url">}]
     append result \n
   } ;# end if
   append result ${::search::javascript}
   append result "  </head>\n"
}

##
# url to show a directory listing to allow browsing
# of files by folder.
# todo:
# button to play folder
# button in insert file/folder into playlist
# todo: Add form controls to define how remote/tracks/sorting is to be used.
###############################################################################
# Function   : dirBrowser::browsePlay
# Description:
# Author     : Tom Wilkason
# Date       : 6/19/2003
###############################################################################
proc dirBrowser::doit {sock {params ""}} {
   Trace
   set myns [uplevel {string trim [namespace current] :}]
   switch -glob -- $myns {
      jukebox* {set control 2}
      remote* {set control 1}
      display* {set control 0}
      default {set control 0}
   }
   upvar #0 Httpd$sock data
   set title "SnackAmp Jukebox"
   set result [leader $title $::snackAmpSettings(cssName)]
   set Tracks [list]
   append result <body>
   set sortMethod "shuffle"
   if {[string length $params]} {
      foreach param [split $params &]  {
         foreach {CmdType Cmd} [split $param =] {break}
         set dCmd [HttpD::Decode $Cmd]
         switch -- $CmdType {
            db_add_start {
               ;# When processing a playlist add, this is the first param
               set playing [LB::active]
               set old_end [LB::end]
               ;# The back reference uses the db_add_start param (which is the params from the
               ;# request that originated the search) which was passed in here. It is saved
               ;# In a hidden field from the prior form.
               append result [AREF "?[NOAMP $dCmd]" Back]
               append result [H2 "Added the following tracks to the playlist"]
               append result "<table class='songPlaylist'>\n"
            }
            db_add {
               ;# When processing a playlist add, these are the middle params, one per item to add
               set track [NOAMP [ROOTTAIL $dCmd]]
               set folder [NOAMP [httpHelpers::displayFolder $dCmd]]
               append result [httpHelpers::tableRow [list $folder $track] headerRow]
               lappend Tracks $dCmd
            }
            db_insert_process {
               ;# When processing a playlist add, this is the final param, signals end of list
               append result "</table>\n"
               append result [H2 "Your requested tracks are now queued to play next"]
               set cp [LB::playing]
               foreach {track} $Tracks {
                  filtering::Insert [incr cp] $track
               } ;# end foreach
               updateFileDisplay
            }
            db_append_process {
               ;# When processing a playlist add, this is the final param, signals end of list
               append result "</table>\n"
               if {[llength $Tracks] > 0} {
                  foreach {track} $Tracks {
                     set next [LB::indexOfFile $track]
                     if {$control==1 || $next==-1 || $next <= [LB::playing] || ($next-[LB::active])>$::snackAmpSettings(minNextTrack)} {
                        LB::userInsert $track
                     }
                  } ;# end foreach
                  append result [H2 "Your requested tracks were queued to  play"]
                  updateFileDisplay
               } else {
                  append result [H2 "No tracks selected or tracks already in playlist"]
               }
            }
            play {
               ;# Play a specific song
               ;# /control?play=some+name.mp3
               set track $dCmd
               if {[catch {filtering::InsertPlay $track} error] } then {
                  saLog $result
                  append result $result
               } else {
                  append result [H2 "Playing '$track'"]
               }
            }
            append {
               set track $dCmd
               set next [LB::indexOfFile $track]
               if {$control==1 || $next==-1 || $next <= [LB::playing] || ($next-[LB::active])>$::snackAmpSettings(minNextTrack)} {
                  ;# Insert after last web insert (instead of end) if possible
                  LB::userInsert $track
#                  append result [H2 "Added $track to Playlist"]
                  updateFileDisplay
               } else {
                  append result [H2 "Skipping $track, it is already in playlist"]
               }
;# TFW: Need to handle duplicates (at least contiguous ones)
            }
            insert {
               set track $dCmd
               append result [H2 "Queued $track to play next"]
               set cp [LB::playing]
               filtering::Insert [incr cp] $track
               updateFileDisplay
;# TFW: Need to handle duplicates (at least contiguous ones)
            }
            delete {
               LB::delete $dCmd
               updateFileDisplay
               ;# Remove from playlist
            }
            folder {
               append result [httpHelpers::getRefreshScript $::snackAmpSettings(refreshRate) "?folder=$Cmd"]
               append result [H2 "Now [string tolower [soundControl::playState]]:"]
               append result [httpHelpers::trackInfo [soundControl::fileName] 1]
               append result {<table><tr><td valign="top">}
               append result [remote/control::nextToPlay $control 10]
               append result {</td><td valign="top">}
               append result [remote/control::queuedToPlay $control 10]
               append result {</td></tr></table>}
               append result [H2 "Select a folder and pick your favorite track"]
               append result [browser::guts $Cmd $control $params]
            }
         } ;# End Switch
      }
   } else {
      append result [httpHelpers::getRefreshScript $::snackAmpSettings(refreshRate) "?folder=[info hostname]"]
      append result [H2 "SnackAmp currently [string tolower [soundControl::playState]]"]
      append result [httpHelpers::trackInfo [soundControl::fileName] 1]
      append result {<table><tr><td valign="top">}
      append result [remote/control::nextToPlay $control 15]
      append result {</td><td valign="top">}
      append result [remote/control::queuedToPlay $control 15]
      append result {</td></tr></table>}
      append result [H2 "Select a folder and pick Your favorite track"]
      append result [browser::guts [info hostname] $control $params]
   }
   append result "</body>\n</html>\n"
   return [list dhtml html [string trim $result]]
}
;##
;# Return the guts of the browser window
;#
proc browser::guts {Cmd control params} {
   # If selected is the toplevel, assume parent is the hostname
   set folder [HttpD::Decode $Cmd]
   array set sides {1 left 2 left}
   set sideNum 1
   set files [list]
   set me  [info hostname]
   append result "<div class='folderDiv'>"
   append result [subst {$httpHelpers::folderopen [AREF "?folder=$me" $me] }]
   if {[string equal $me $folder]} {
      set parent $me
      set folders $::snackAmpSettings(mediaDirs)
      append result "<br>\n"
   } else {
      set folders [list]
      ##
      # Display the combo box with this folders siblings with this
      # folder selected
      #
      if {[lsearch $::snackAmpSettings(mediaDirs) $folder] >= 0} {
         set parent $folder
      } else {
         set parent [file dirname $folder]
      }
      ;# Create back track in proper order
      set tlevel $parent
      set dirPath {}

      while {1} {
         set dirPath [linsert $dirPath 0 $tlevel]
         set under [file dirname $tlevel]
         if {[lsearch $::snackAmpSettings(mediaDirs) $tlevel] >= 0} {
            break
         } elseif {[string equal $tlevel $under]} {
            break
         } else {
            set tlevel $under
         }
      }
      ##
      # Create a back path to the root folder from selected folder
      #
      foreach {tlevel} $dirPath {
         append result [subst {<b></b> [AREF "?folder=[ENCODE $tlevel]" [TAILIFCAN $tlevel]] }]
      }
      append result "<b></b> "
#      append result "<br>\n"
      ##
      # Put the siblings of parent in a combo box
      # Build up path, higher to lower order
      #
if {1} {
      ;# No combo box
      append result [subst {[AREF "?folder=[ENCODE $folder]" [TAILIFCAN $folder]]<br>}]
} else {
      set siblings [lsort -dictionary [glob -nocomplain -types d -directory $parent *]]
      if {[llength $siblings] > 0} {
         append result [subst {
            <select size="1" name="folder" class="formText"  onChange="javascript:window.location.href='?folder=' + escape(options\[selectedIndex\].value);">
         }]
         set tfolder [file tail $folder]
         foreach {item} $siblings {
            set item [file tail $item]
            if {[string equal $tfolder $item]} {
               set temp [subst {<option selected value="[file join $parent $item]">$item</option>\n}]
            } else {
               set temp [subst {<option value="[file join $parent $item]">$item</option>\n}]
            }
            append result [NOAMP $temp]
         }
         append result {
            </select><br>
         }
      }
}
      ##
      # Build up a list of all the folders and files in the two tables, folders then tracks
      # TFW: cache the files and folders, fire after time to unset them later
      set contents [lsort -dictionary [glob -nocomplain -directory $folder *]]
      foreach entry $contents {
         if {[file isdirectory $entry]} {
            lappend folders $entry
         } elseif {[info exists ::taExtensions([file extension $entry])]} {
            lappend files $entry
         }
      } ;# end foreach
      set folders [lsort -unique $folders]
   }
   ##
   # Show the sub-folders in a table along with coverart if possible
   #
   if {[llength $folders]} {
      incr sideNum
      set tail [file tail $folder]
      ;# Find minimum folder length for indent offsets
      foreach item $folders {
         ;# TFW: Will want to add a checkbutton next to these as well for streaming
#         set cov [httpHelpers::coverArt $item]
#         if {[string length $cov]} {
#            set ca [AREF "?folder=[ENCODE $item]" $cov]
#            append ca <br>
#         } else {
            set ca ""
#         }
         set link [subst {[SP]$httpHelpers::folderclosed [AREF "?folder=[ENCODE $item]" [TAILIFCAN $item]]<br>\n}]
         append result [subst {<tr><td>$ca$link</td></tr>}]
      } ;# end foreach
   }
   append result "</div>"
   ##
   # Show the files in a table
   # TFW: This should be in a form with each item with a checkbox to add multiple
   # ones to the playlist (like search results)
   # TFW: The table needs alternating rows of color, with no border. Can use styles
   if {[llength $files]} {
      append result "<div class='fileDiv''>"
      if {$control==1} {
         append result [subst {<FORM METHOD="POST" ACTION="" NAME="jb_add_form">}]
         append result "<input type=hidden name=db_add_start value=\"$params\">"
      }
      append result [httpHelpers::coverArt $folder]
      append result "<font size='5'>[file tail $folder]</font>"
      append result "<table class='songPlaylist'>"
      foreach file $files {
         set bgc [httpHelpers::rowStyle i]
         # Need to strip out prior append=params (no longer needed with popup)
         # regsub {(append=.*?)&} $params  {} params
         set cb "<INPUT TYPE=checkbox NAME='db_add' VALUE=\"$file\">"
         set ftail [AREF "?append=[ENCODE $file]&folder=$Cmd" [TAILIFCAN $file] "Queue Track to Play"]
         if {$control==1} {
            set buttonLink [httpHelpers::popupInfo $file]
            set trImage [httpHelpers::popUpProcess play "" $file]
            append result [httpHelpers::tableRow [list $cb $buttonLink "$trImage $ftail"] $bgc]
         } else {
            set trImage $httpHelpers::trackimage
            append result [httpHelpers::tableRow [list "$trImage $ftail"] $bgc]
         }
      } ;# end foreach
      append result "</table>\n"
      if {$control==1} {
         append result "<INPUT TYPE=submit VALUE=Append NAME=db_append_process>\n"
         append result "<INPUT TYPE=submit VALUE=Insert NAME=db_insert_process>\n"
         append result "</FORM>\n"
         append result [subst {[AREF "javascript:toggleAll_jb_add(true)" "Select All"] | [AREF "javascript:toggleAll_jb_add(false)" "Unselect All"]<br>}]
      }
      append result "</div>"
   } else {
   #                  append result "<hr>\n"
   #                  append result [H2 "No tracks within [file tail $folder]"]
   }
   return $result
}
##################################################################################
#                             g e t p l a y l i s t
##################################################################################
##
# playlist namespace, returns a nicly formatted list of playlist and other
# to build up streaming playlists.
#
namespace eval streaming/getplaylist.pls {
   proc init { } {
     return 1
   }
}
##
# Playlist code, from a web browser
# Processes a request to play a track, playlist, active playlist or auto-playlist
# Creates a winamp type pls dynamic file that when processed by the /streaming/live command
# will create a queue of songs then play them.
# Note: Add ability to send the entire stream length down as icy data. Good for tracks
##
# Note: TFW: Add another option to sort, shuffle or shufflesort
#            pass it on to the /streaming/live parameter, the sort param
#            should be specified first, otherwise it will be
#            ignored.
proc streaming/getplaylist.pls::process {sock {params ""}} {
   Trace
   upvar #0 Httpd$sock data
   set result ""
   set sortMethod "shuffle"
   set how "stream"
   set param ""
   set method "unknown"
   set max 50
   set refer $::snackAmpSettings(internalWebBase)
   if {[string length $params]} {
      foreach pr [split $params &]  {
         foreach {CmdType Cmd} [split $pr =] {break}
         ;# Create a dynamic playlist
         switch -- $CmdType {
            active -
            playlist -
            track -
            folder -
            apl {
               set method $CmdType
               set param $Cmd
            }
            tree {
               switch -- $Cmd {
                  no {set fmethod folder}
                  yes {set fmethod treefolder}
               } ;# End Switch
            }
            sort {set sortMethod [HttpD::Decode $Cmd]}
            how  {set how [HttpD::Decode $Cmd]}
            submit  {}
            default {saLog "streaming/getplaylist.pls::process: Invalid subcommand - '$CmdType'"}
         }
      }
      ##
      # We have all the params, now process the options
      #
      switch -- [string tolower $how] {
         stream {
            ;# Create a stream of multiple tracks (except track option)
            switch -- $method {
               active {
                  set    result "File1=$refer/streaming/live?sort=$sortMethod&active=$param\n"
                  append result "Title1=Active Playlist"
               }
               apl {
                  set    result "File1=$refer/streaming/live?sort=$sortMethod&apl=$param\n"
                  append result "Title1=Auto Playlist [HttpD::Decode $param]"
               }
               playlist {
                  set    result "File1=$refer/streaming/live?sort=$sortMethod&playlist=$param\n"
                  append result "Title1=Playlist [HttpD::Decode $param]"
               }
               treefolder -
               folder {
                  set    result "File1=$refer/streaming/live?sort=$sortMethod&$fmethod=$param\n"
                  append result "Title1=Track [HttpD::Decode $param]"
               }
               track {
                  set    result "File1=$refer/streaming/live?track=$param\n"
                  append result "Title1=Track [HttpD::Decode $param]"
               }
               default {saLog "streaming/getplaylist.pls::process: Invalid subcommand method=$method"}
            }
            #puts "stream:$result"
         }
         tracks {
            ;# Create individual tracks
            switch -- $method {
               active {
                  if {[catch {streaming/live::activeplaylist $sortMethod} tracks]} then {
                     saLog $tracks
                  } else {
                     set i 1
                     set result ""
                     foreach index $tracks {
                        set track [lindex [catalog::sortedList] $index]
                        append result "File$i=$refer/streaming/live?track=[ENCODE $track]\n"
                        append result "Title$i=[LB::getListName $track]\n"
                        incr i;if {$i > $max} {break}
                     }
                  }
               }
               apl {
                  if {[catch {streaming/live::apl [HttpD::Decode $param] $sortMethod} tracks] } then {
                     saLog $tracks
                  } else {
                     set i 1
                     set result ""
                     foreach index $tracks {
                        set track [lindex [catalog::sortedList] $index]
                        append result "File$i=$refer/streaming/live?track=[ENCODE $track]\n"
                        append result "Title$i=[LB::getListName $track]\n"
                        incr i;if {$i > $max} {break}
                     }
                  }
               }
               playlist {
                  if {[catch {streaming/live::playlist [HttpD::Decode $param] $sortMethod} tracks]} then {
                     saLog $tracks
                  } else {
                     set i 1
                     set result ""
                     ;# shuffle
                     foreach index $tracks {
                        set track [lindex [catalog::sortedList] $index]
                        append result "File$i=$refer/streaming/live?track=[ENCODE $track]\n"
                        append result "Title$i=[LB::getListName $track]\n"
                        incr i;if {$i > $max} {break}
                     }
                  }
               }
               folder {
                  set i 1
                  set result ""
                  set path [HttpD::Decode $param]
                  set files [lsort -dictionary [glob -nocomplain -directory $path *]]
                  foreach track $files {
#puts "get playlist folder $track"
                     if {[info exists ::taExtensions([file extension $track])]} {
                        append result "File$i=$refer/streaming/live?track=[ENCODE $track]\n"
                        append result "Title$i=[LB::getListName $track]\n"
                        incr i;if {$i > $max} {break}
                     }
                  } ;# end foreach
               }
               treefolder {
                  set i 1
                  set result ""
                  set path [HttpD::Decode $param]
                  set tracks [catalog::filesMatchingPattern [file join $path *]]
                  foreach track $tracks {
#puts "treefolder $track"
                     append result "File$i=$refer/streaming/live?track=[ENCODE $track]\n"
                     append result "Title$i=[LB::getListName $track]\n"
                     incr i;if {$i > $max} {break}
                  }
               }
               track {
                  set    result "File1=$refer/streaming/live?track=$param\n"
                  append result "Title1=Track [LB::getListName [HttpD::Decode $param]]"
               }
               default {saLog "streaming/getplaylist.pls::process: Invalid subcommand method=$method"}
            }
         }
         default {saLog "streaming/getplaylist.pls::process: Invalid subcommand how=$how"}
      }
#puts "playlist=$result"
      return [list dhtml pls [string trim $result]]
   } else {
      set title "SnackAmp Jukebox"
      set result [remote/control::leader $title $::snackAmpSettings(cssName)]
      append result [H1 "Please specify a parameter"]
      append result "</body>\n</html>"
      return [list dhtml html [string trim $result]]
   }
}
##################################################################################
#                              l i v e
##################################################################################
###############################################################################
# Function   : streaming/live namespace
# Description: Process streaming files
# Author     : Tom Wilkason
# Date       : 3/13/2003
###############################################################################
namespace eval streaming/live {
   proc init {args} {
      return 1
   }
}
##
# the /streaming/live process method just retuns a file name from an existing file.
# used to initiate a stream to a client. This is run from shortcuts
# AFTER the playlist has been built and selected.
# Note: TFW: Add another option to sort, shuffle or sort/shuffle
#
proc streaming/live::process {sock {params ""}} {
   Trace
   upvar #0 Httpd$sock data
   set Tracks [list]
   set sortMethod "shuffle"
   if {[string length $params]} {
      foreach param [split $params &]  {
         foreach {CmdType Cmd} [split $param =] {break}
         set Cmd [HttpD::Decode $Cmd]
#puts "streaming/live::process $CmdType=$Cmd"

         switch -- $CmdType {
            apl {
               ;# Get some apl file list of indicies
               if {[catch {streaming/live::apl $Cmd $sortMethod} result] } then {
                  return -code error $result
               } else {
                  if {[llength $result] > 0} {
                     set data(streaming) $result
                     set data(streamingLength) 44000000
                     set data(Title) "AutoPlaylist From $data(mime,host)"
                     set data(Album) "$Cmd ([llength $result] trks)"
                     return [list file html [streamBuffer::popFile $sock]]
                  } else {
                     return -code error "No apl files for '$Cmd'"
                  }
               }
            }
            playlist {
               ;# Get some tracks from a playlist
               if {[catch {streaming/live::playlist $Cmd $sortMethod} result]} then {
                  return -code error $result
               } else {
                  ;##
                  ;# Don't return track, return a m3u file
                  ;#
                  if {[llength $result] > 0} {
                     set data(streaming) $result
                     set data(streamingLength) 44000000
                     set data(Title) "Playlist From $data(mime,host)"
                     set data(Album) "$Cmd ([llength $result] trks)"
                     return [list file html [streamBuffer::popFile $sock]]
                  } else {
                     return -code error "No playlist files for '$Cmd'"
                  }
               }
            }
            active {
               ;# Get some tracks from a playlist
               if {[catch {streaming/live::activeplaylist $sortMethod} result]} then {
                  return -code error $result
               } else {
                  ;##
                  ;# Don't return track, return a m3u file
                  ;#
                  if {[llength $result] > 0} {
                     set data(streaming) $result
                     set data(streamingLength) 44000000
                     set data(Title) "Active Playlist From $data(mime,host)"
                     set data(Album) "$Cmd ([llength $result] trks)"
                     return [list file html [streamBuffer::popFile $sock]]
                  } else {
                     return -code error "No active playlist files for '$Cmd'"
                  }
               }
            }
            folder {
#puts "streaming/live folder $Cmd"
               set result [list]
               set files [sortByMethod [glob -nocomplain -directory $Cmd *] $sortMethod]
               foreach file $files {
                  if {[info exists ::taExtensions([file extension $file])]} {
                     set loc [catalog::whereInDb $file]
                     if {$loc >= 0} {
                        lappend result $loc
                     }
                  }
               } ;# end foreach

               if {[llength $result] > 0} {
                  set data(streaming) $result
                  set data(streamingLength) 44000000
                  set data(Title) "Folder $Cmd From $data(mime,host)"
                  set data(Album) "$Cmd ([llength $result] trks)"
                  return [list file html [streamBuffer::popFile $sock]]
               } else {
                  return -code error "No folder files found for '$Cmd'"
               }
            }
            treefolder {
#puts "treefolder $Cmd"
               set files [sortByMethod [catalog::filesMatchingPattern [file join $Cmd *]] $sortMethod]
               set result [list]
               foreach file $files {
                  set loc [catalog::whereInDb $file]
                  if {$loc >= 0} {
                     lappend result $loc
                  }
               } ;# end foreach

               ;##
               ;# Don't return track, return a m3u file
               ;#
               if {[llength $result] > 0} {
                  set data(streaming) $result
                  set data(streamingLength) 44000000
                  set data(Title) "Treefolder $Cmd From $data(mime,host)"
                  set data(Album) "$Cmd ([llength $result] trks)"
                  return [list file html [streamBuffer::popFile $sock]]
               } else {
                  return -code error "No folder files found for '$Cmd'"
               }
            }
            track {
               ;# Process a single track in a playlist
               set loc [catalog::whereInDb $Cmd]
               if {$loc >= 0} {
                  set file [lindex [catalog::sortedList] $loc]
                  set data(streaming) $loc
                  set data(streamingLength) [file size $file]

                  ;##
                  ;# How do we configure time to this?
                  ;#

                  set tempSound [snack::sound -debug 0 -file $file]
                  if {[catch {id3Tag::songDetails $file s Details} error] } then {
                     saLog $error
                  } else {
                     ;# Fill in data that will go into icy-meta tags
                     foreach {index} {Title Album Artist Genre rate} {
                        if {[info exists Details($index)]} {
                           set data($index) $Details($index)
                        } else {
                           set data($index) ""
                        }
                     }
                     # add bitrate here
                  }
                  $tempSound destroy
                  return [list file html [streamBuffer::popFile $sock]]
               }
            }
            sort        {set sortMethod $Cmd}
            default {
               return -code error "streaming/live::process: Invalid subcommand - '$CmdType'"
            }
         } ;# End Switch
      }
   } else {
      return -code error "streaming/live::process: No Parameter '$params'"
   }
}
##
#  Return a list of indices to auto playlist files
#
proc streaming/live::apl {name sortMethod} {
   Trace
   set fName [file join $::HOME $name.tafs]
   if {![file exists $fName]} {
      return -code error "$fName does not exist"
   } else {
      if {[catch {open $fName r} fid]} then {
         return -code error $fid
      }
      fconfigure $fid -buffersize 100000
      set full [split [read $fid] \n]
      close $fid
      ;# Get folder list
      set biglist [list]
      array set myte [array get ::teFolders]
      foreach line $full {
         foreach {Index Value} $line {
            set myte($Index) $Value
         }
      } ;# end foreach
      set i 0
      ;# Grab locations in an array and shuffle it
      set tracks [sortByMethod [catalog::sortedList] $sortMethod]
      foreach track $tracks {
         set trackDir [file dirname $track]
         if {[info exists myte($trackDir)]} {
            if {$myte($trackDir)==1} {
               lappend biglist $i
            }
         } else {
            #saLog "Couldn't find '$trackDir' as an index"
         }
         incr i
      }
   }
   return $biglist
}
##
# Return a list of indicies for a playlist
#
proc streaming/live::playlist {name sortMethod} {
   Trace
   set name [string trim $name \"] ;#"
   foreach ext {m3u taf} {
      set fName [file join $::HOME $name.$ext]
      if {[file exists $fName]} {
         break
      }
   } ;# end foreach
   if {![file exists $fName]} {
      saLog "streaming/live::playlist: $fName does not exist"
      return -code error "$fName does not exist"
   } else {

      ;# Here we have a file case issue
      if {[catch {open $fName r} fid]} then {
         return -code error $fid
      }
      fconfigure $fid -buffersize 100000
      set full [split [read $fid] \n]

      close $fid
      set tracks [sortByMethod $full $sortMethod]
      ;# Get folder list
      set biglist [list]
      foreach line $tracks {
         set line [file join $line] ;# make tcl name
         set loc [catalog::whereInDb $line]
         if {$loc >= 0} {
            lappend biglist $loc
         } else {
 #           saLog "$line not in database"
         }
      }
   }
   return $biglist
}
;##
;# Return a list of indicies to current playlist on GUI
;#
proc streaming/live::activeplaylist {sortMethod} {
   set biglist [list]
   ;##
   ;# Note: May want to get from cursor foward.
   ;#
   set tracks [sortByMethod [LB::getAllNames 1] $sortMethod]
   foreach file $tracks {
      set loc [catalog::whereInDb $file]
      if {$loc >= 0} {
         lappend biglist $loc
      }
   }
   return $biglist
}
proc streaming/live::folder {name} {
}
proc streaming/live::recurfolder {name} {
}
###############################################################################
# Function   : sortByMethod
# Description: Sort a list by the appropriate method
# Author     : Tom Wilkason
# Date       : 3/24/2003
###############################################################################
proc streaming/live::sortByMethod {list how} {
   switch -glob -- [string tolower $how] {
      sort*tracks     {return [lsort -dictionary $list]}
      shuffle*tracks  {return [shuffle $list]}
      shuffle*sort    {return [shuffleByFolder $list ldsort]}
      shuffle*shuffle {return [shuffleByFolder $list shuffle]}
      none {return $list}
   } ;# End Switch
   return $list
}

##################################################################################
#                              s t r e a m i n g  f i l e  b r o w s e r
##################################################################################
namespace eval streaming/browser {
   proc init { } {
     return $::snackAmpSettings(EnableStreaming)
   }
   variable brform {
  <H2>Enter Play Folder Option</H2>
  <form name=option_form>
  <TABLE>
    <TR>
      <TD>Sorting</TD>
      <TD>Streaming</TD>
      <TD>Subfolders</TD>
    </TR>
    <TR>
     <TD>
       <select size="1" name="cmb_Sort" class="formText">
         <option>Sort-Tracks</option>
         <option selected>Shuffle-Tracks</option>
         <option>Shuffle->Sort</option>
         <option>Shuffle->Shuffle</option>
       </select>
     </TD>
     <TD>
       <select size="1" name="cmb_How" class="formText">
         <option selected>stream</option>
         <option>tracks</option>
       </select>
     </TD>
     <TD>
       <select size="1" name="cmb_Sub" class="formText">
         <option selected>no</option>
         <option>yes</option>
       </select>
     </TD>
    </TR>
  </TABLE>
  </form>
   }
   variable foldersel {
  <script language="JavaScript">
   function popupInfo(url,height,width,tag) {
     var prefs = "scrollbars=yes,menubars=no,toolbars=no,"
     + "resizable=yes,left=0,top=0,height="+height+",width="+width;
     window.open(url, tag, prefs);
   }
   // getselected: Return the selected item from a combo box
   // TFW: element as object doesn't work due to ?? no type ??
   function getselected(element,len) {
      var i;
      for (i==0; i<len; i++) {
         alert(element[i].selected + ' ' + element[i].text);
         if (element[i].selected) {
            return element[i].text;
         }
      }
      return 'none';
   }
   function foldersel(url,folder) {
     var useurl='none';
     var selsort='no sort';
     var selhow='no stream';
     var selsub='no';
     var i=0;
     //stream
     //selhow = getselected(document.option_form.cmb_How,2);
     for (i==0;i<2;i++) {
        if (document.option_form.cmb_How[i].selected) {
           selhow=document.option_form.cmb_How[i].text;
           break;
        }
     }

     //subfolders
     for (i==0;i<2;i++) {
        if (document.option_form.cmb_Sub[i].selected) {
           selsub=document.option_form.cmb_Sub[i].text;
           break;
        }
     }
     //sort
     for (i==0;i<4;i++) {
        if (document.option_form.cmb_Sort[i].selected) {
           selsort=document.option_form.cmb_Sort[i].text;
           break;
        }
     }
     useurl = url+
       '/?sort='  + selsort +
       '&tree='   + selsub  +
       '&how='    + selhow  +
       '&folder=' + folder;
     window.open(useurl,"tinyWindow", 'width=100,height=100');
   }
</script>
  }
}
proc streaming/browser::leader {title css {url "xx"}} {
   Trace
   variable foldersel
   ;# title is used in subst
   append result [subst ${remote/control::head}]
   if {![string equal $url "xx"]} {
     append result \n
     append result [subst {    <META HTTP-EQUIV="Refresh" CONTENT="$::snackAmpSettings(refreshRate);URL=$url">}]
     append result \n
   } ;# end if
   append result $foldersel
   append result "  </head>\n"
}

##
# url to show a directory listing to allow browsing
# of files by folder.
# todo:
# button to play folder
# button in insert file/folder into playlist
# todo: Add form controls to define how streaming/tracks/sorting is to be used.
proc streaming/browser::process {sock {params ""}} {
   Trace
   variable brform
   upvar #0 Httpd$sock data
   set title "SnackAmp Jukebox"
   set result [leader $title $::snackAmpSettings(cssName)]
   append result <body>
   set sortMethod "shuffle"
   if {[string length $params]} {
      foreach param [split $params &]  {
         foreach {CmdType Cmd} [split $param =] {break}
         switch -- $CmdType {
            sort {
               set sortMethod $Cmd
            }
            folder {
               append result [H1 "SnackAmp media folders on [info hostname]"]
               set folders [list]
               set files [list]
               set folder [HttpD::Decode $Cmd]
               set contents [lsort -dictionary [glob -nocomplain -directory $folder *]]
               foreach entry $contents {
                  if {[file isdirectory $entry]} {
                     lappend folders $entry
                  } elseif {[info exists ::taExtensions([file extension $entry])]} {
                     lappend files $entry
                  }
               } ;# end foreach
               #----------------------------
               ;##
               ;# Create a list of all folders, includling toplevel ones
               ;#
               set fldr $folder
               while {1} {
                  ;# Get list of all parent folders
                  set under [file dirname $fldr]
                  if {[string equal $under $fldr]} {
                     break
                  } else {
                     lappend folders $fldr ;########
                     set fldr $under
                  }
               }
               ;# List of toplevel folder
               foreach item $::snackAmpSettings(mediaDirs) {
                  lappend folders $item
                  if {[string match "$item*" $folder]} {
                     set icon $httpHelpers::folderopen
                  } else {
                     set icon $httpHelpers::folderclosed
                  }
                  append result [subst {$icon <a href="?folder=[ENCODE $item]">$item</a><br>\n}]
               } ;# end foreach
               #----------------------------
               set folders [lsort -unique $folders]
               ;# Show the folders
               if {[llength $folders]} {
                  append result "<hr>\n"
                  append result $brform
                  append result [H2 "Current folder $folder"]
                  append result "<table class='songPlaylist'>"
                  ;# Find minimum folder length for indent offsets
                  foreach item $folders {
                     lappend reps [llength [file split $item]]
                  }
                  set minrep [lindex [lsort -integer $reps] 0]
                  foreach item $folders {
                     set icon $httpHelpers::folderclosed
                     if {[string equal $item $folder]} {
                        set icon $httpHelpers::folderopen
                     }
                     set rep [expr {[llength [file split $item]]-$minrep}]
                     set link "[string repeat [SP] $rep]$icon <a href=\"?sort=$sortMethod&folder=[ENCODE $item]\">$item</a><br>\n"
                     ;# S = Stream folder contents
                     ;# T = Tracks for folder contents
                     set base "/streaming/getplaylist.pls"
                     set F [subst {<a href="javascript:foldersel('$base','[ESCAPE $item]')" title="Play Track Immediately">$httpHelpers::playbutton</A>}]
                     append result [httpHelpers::tableRow [list $F $link] darkRows]
                  } ;# end foreach
                  append result "</table>\n"
               }
               ;# Show the files
               if {[llength $files]} {
                  append result "<hr>\n"
                  append result [httpHelpers::coverArt $folder]
                  append result "<table class='songPlaylist'>"
                  foreach file $files {
                     set I [httpHelpers::popupInfo $file]
                     set link [AREF "/streaming/getplaylist.pls?sort=$sortMethod&track=[ENCODE $file]" $httpHelpers::playbutton "Play Track Immediately"]<br>
                     set size "[commify [expr {[file size $file]/1024}]] KB"
                     append result [httpHelpers::tableRow [list $I $link [file tail $file] $size] [httpHelpers::rowStyle i]]
                  } ;# end foreach
                  append result "</table>\n"
               } else {
                  append result "<hr>\n"
                  append result [H1 "No tracks within $folder"]
               }
            }
         } ;# End Switch
      }
   } else {
      append result [H1 "SnackAmp folder browser"]
      append result [H2 "Please select a folder below to browse"]
      foreach folder $::snackAmpSettings(mediaDirs) {
         append result $httpHelpers::folderclosed[AREF "?folder=[ENCODE $folder]" $folder]<br>
      } ;# end foreach
   }
   append result "</body>\n</html>\n"
   return [list dhtml html [string trim $result]]
}
##################################################################################
#                 s t r e a m i n g   p l a y l i s t  e x a m p l e
##################################################################################
##
# Example code to display a selection of sources on this server
#
namespace eval streaming/playlists {
   proc init { } {
     return 1
   }
}
proc streaming/playlists::process {sock {params ""}} {
   Trace
   upvar #0 Httpd$sock data
   ;##
   ;# This can be prettied up to select the various
   ;# playlists that are created.
   ;# Can add options to use sorting options
   ;#
   set title "SnackAmp Jukebox"
   set result [remote/control::leader $title $::snackAmpSettings(cssName)]
   append result <body>
   ##
   # Need to add code to show folder trees
   #
   append result {[H1 <a href='/streaming/getplaylist.pls?active=active'>Current playlist</a>]}
   set Files [catalog::getAutoPlaylists]
   ;# Create option list of all auto-playlist files
   if {[llength $Files] > 0} {
      append result [H1 "Auto-Playlist files"]
      foreach {file} $Files {
         set choice [ROOTTAIL $file]
         append result [AREF "/streaming/getplaylist.pls?apl=[ENCODE $choice]" $choice]<br>
      } ;# end foreach
   }
   ;# Create option list of all playlist files
   set Files [catalog::getPlaylists]
   if {[llength $Files] > 0} {
      append result [H1 "Playlist files"]
      foreach {file} $Files {
         set choice [ROOTTAIL $file]
         append result [AREF "/streaming/getplaylist.pls?playlist=[ENCODE $choice]" $choice]<br>
      } ;# end foreach
   }
   append result "</body>\n</html>\n"
   return [list dhtml html [string trim $result]]
}
###############################################################################
# Function   : search namespace
# Description: Handles interface to request songs to be added to playlist
#              Can be called from different URL types and reacts accordingly
# Author     : Tom Wilkason
###############################################################################
namespace eval search {
   variable myns
   variable control
   variable search_form {
    <!-- Search Database -->
    <table border="1" width="100%" bordercolor="darkblue" cellspacing="0" cellpadding="0">
      <tr>
        <td width="100%" bgcolor="darkblue" align="center"><font color="white" face="Helvetica">
        <b>Search All Tracks for Matching Text</b></font></td>
      </tr>
      <tr>
        <td width="100%" align="left">
          <form method="POST" action="/search" TITLE="Enter text then hit Search to find all songs that contain the text">
            <input type="submit" value="Search" name="Button">
            <font face="Helvetica">
              <input type="text" name="jb_search" size="30">
            </font>
          </form>
        </td>
      </tr>
    </table>
    <p>
   }
   variable close_window {<a href="" onClick="window.close()">Click this to close the window</A>}
   ;##
   ;# Javascript helper routines
   ;#
   variable javascript {
     <script language="JavaScript">
     //<!--
        function popupInfo(url,height,width,tag) {
           var prefs = "scrollbars=yes,menubars=no,toolbars=no,"
           + "resizable=yes,left=0,top=0,height="+height+",width="+width;
           window.open(url, tag, prefs);
        }
        function textWindow(url) {
           var prefs = "scrollbars=no,menubars=no,toolbars=no,"
           + "resizable=yes,left=100,top=100,height=20,width=500;"
           tWin=window.open(url, "Text", prefs);
           setTimeout("tWin.close()",1000)
        }
        function toggleAll_jb_add(checked) {
           len = document.jb_add_form.elements.length;
           var i;
           for (i = 0; i < len; i++) {
              document.jb_add_form.elements[i].checked = checked;
           }
        }
        function toggleAll_jb_search_pl(checked) {
           len = document.jb_search_pl_form.elements.length;
           var i;
           for (i = 0; i < len; i++) {
              document.jb_search_pl_form.elements[i].checked = checked;
           }
        }
     //-->
     </script>
   }
   proc init { } {
     return 1
   }
}
###############################################################################
# Function   : search::doit state machine
# Description: Main function that is called by web server when /search
#              is entered.
#              e.g. /search?jb_search_pl=*abc*
# Author     : Tom Wilkason
# Date       : 12/15/2002
###############################################################################

;# TFW: /search is used now but search needs to be called as /remote/search or /jukebox/search
proc search::doit {sock {params ""}} {
   Trace
   variable search_form
   variable javascript
   upvar #0 Httpd$sock data
   variable myns [uplevel {string trim [namespace current] :}]
   set title "SnackAmp Jukebox"
   set css $::snackAmpSettings(cssName)
   append Result [subst ${remote/control::head}]
   append Result $javascript
   append Result "  </head>\n<body>\n"
   append Result [subst {<a href="/$myns?$params">Refresh</a>|}]
   append Result ${search::close_window}
   switch -glob -- $myns {
      jukebox* {
         set cont 0
      }
      remote* {
         set cont 1
      }
      default {}
   }
   variable control $cont
   ;# Appropriate link to browser url e.g. /jukebox/browser
   set myBrowser [file join [file dirname $myns] browser]
   ;##
   ;# Need to have search_form optional
   ;#
   #append Result [subst $search_form]
   set Search_What pl
   set Tracks [list]
   foreach param [split $params &]  {
      foreach {CmdType Cmd} [split $param =] {break}
      set Cmd [HttpD::Decode $Cmd]
      switch -- $CmdType {
         jb_search {
            ;# stage 1
            set filter $Cmd
         }
         search_what {
            ;# stage 2
            switch -glob -- [string tolower $Cmd] {
               "all*files" {
                  ;# User specified text to search catalog
                  append Result [H2 "The following tracks matched your search criteria '$filter'"]
                  append Result [H2 "Please check desired tracks and press <i>Append</i> or <i>Insert</i> at the bottom to add them to the playlist"]
                  ;# Pass params for putting in back button on next form
                  append Result [search::searchCatalog $myBrowser $filter [expr {$::snackAmpSettings(httpNextLength)*5}] $params]
               }
               "play*" {
                  ;# User specifed some text to search within the playlist
                  ;# This goes in a pop-up
                  append Result [H2 "The following tracks from the Play List matched your search criteria '$filter'"]
                  append Result [search::searchPlaylist $myBrowser $filter 0]
               }
               "auto*" {
                  ;# User specifed some text to search within the playlist
                  ;# This goes in a pop-up
                  append Result [H2 "The following tracks from the Auto-Playlist matched your search criteria '$filter'"]
                  append Result [search::searchPlaylist $myBrowser $filter 1]
               }
            } ;# End Switch
         }
         play {
            ;# Play a specific song
            ;# /control?play=some+name.mp3
            if {[catch {filtering::InsertPlay $Cmd} error] } then {
               saLog $result
               append Result $result
            }
         }
         delete {
            LB::delete $Cmd
            updateFileDisplay
            ;# Remove from playlist
         }
         question {
            ;# This goes in a pop-up
            ;# javascript pop-up track details (like playinfo)
            if {![file exists $Cmd]} {
               ;# TFW: TF Lookup needs to be wrapped in LB
               set Cmd [LB::getFullNameFromHash $Cmd 1]
            } ;# end if
            ;# TFW: would be nice to make the fonts smaller
            append Result [httpHelpers::trackInfo $Cmd]
         }
         jb_add_start {
            ;# When processing a playlist add, this is the first param
            set playing [LB::active]
            set old_end [LB::end]
            ;# The back reference uses the jb_add_start param (which is the params from the
            ;# request that originated the search) which was passed in here. It is saved
            ;# In a hidden field from the prior form.
            append Result |[AREF "/$myns?$Cmd" Back]\n
            append Result [H2 "Added the following tracks to the playlist"]
            append Result "<table class='songPlaylist'>\n"
         }
         jb_add {
            ;# When processing a playlist add, these are the middle params, one per item to add
            set next [LB::indexOfFile $Cmd]
            if {$control==1 || $next==-1 || $next<=$playing} {
               set track [NOAMP [file root [file tail $Cmd]]]
               set folder [NOAMP [httpHelpers::displayFolder $Cmd]]
               append Result [httpHelpers::tableRow [list $folder $track] headerRow]
               lappend Tracks $Cmd
            }
         }
         jb_append_process {
            ;# When processing a playlist append, this is the final param, signals end of list
            append Result "</table>\n"
            if {[llength $Tracks] > 0} {
               append Result [H2 "Your requested tracks were queued to play"]
               foreach {track} $Tracks {
                  LB::userInsert $track
               } ;# end foreach
               updateFileDisplay
            } else {
               append Result [H2 "No tracks selected or tracks already in playlist"]
            }
         }
         jb_insert_process {
            ;# When processing a playlist insert, this is the final param, signals end of list
            append Result "</table>\n"
            if {[llength $Tracks] > 0} {
               append Result [H2 "Your requested tracks are now queued to play next"]
               set cp [LB::playing]
               foreach {track} $Tracks {
                  filtering::Insert [incr cp] $track
               } ;# end foreach
               updateFileDisplay
            } else {
               append Result [H2 "No tracks selected or tracks already in playlist"]
            }
         }
      }
   }
   ;# Insert Display Playlist link
   append Result "</body>\n</html>"
   return [list dhtml html $Result]
   return
}
###############################################################################
# Function   : getAPLFileList
# Description: Return a list of all APL files
# Author     : Tom Wilkason
# Date       : 12/24/2002
###############################################################################
proc search::getAPLFileList {args} {
   variable control
   variable myns
   #set RF $snackAmpSettings(UseRemovableFolders)
   #set rmtFilter [file join $::RMT *]
   ;# TFW: Add proper support for removable media here
   ;##
   set result [list]
   foreach {File} [catalog::sortedList] {
      if {[filtering::checkIfIncluded [file dirname $File]]} {
         lappend result $File
      }
   }
   return $result
}
###############################################################################
# Function   : searchPlaylist
# Description: Based on a pattern from the web entry box, filter the current
#              playlist and return the matching results
#
# Author     : Tom Wilkason
###############################################################################
proc search::searchPlaylist {parNs filter {aplOnly}} {
   variable control
   variable myns
   set i 0
   set result {<table class='songPlaylist'>}
   if {$aplOnly} {
      set files [search::getAPLFileList]
   } else {
      set files [LB::getAllNames 1]
   } ;# end if
   if {$control} {
      append result [httpHelpers::tableRow [list Num ? P X Folder "Track Name"] headerRow]
   } else {
      append result [httpHelpers::tableRow [list Num ? Folder "Track Name"] headerRow]
   }
   foreach {file} $files {
      if {[string match -nocase *$filter* $file]} {
         set bgc [httpHelpers::rowStyle j 5]
         set track [ROOTTAIL $file]
         set folder [httpHelpers::displayFolder $file]
         if {$folder==""} {
            set flink [SP]
         } else {
            set flink [AREF "/$parNs?folder=[ENCODE [file dirname $file]]" $folder]
         }
         set Info [httpHelpers::popupInfo $file]
         ;# The second param just performs another search using the same filter
         if {$control} {
            set Play [subst {<a href="/$myns?play=[ENCODE $file]&jb_search=$filter&search_what=Playlist" title="Play Track Immediately">$httpHelpers::playbutton</a>}]
            set Delete [subst {<a href="/$myns?delete=$i&jb_search=$filter&search_what=Playlist">$httpHelpers::deleteimage</a>}]
            append result [httpHelpers::tableRow [list $i $Info $Play $Delete $flink $track] $bgc]
         }  else {
            append result [httpHelpers::tableRow [list $i $Info $flink $track] $bgc]
         }
      } ;# end if
      incr i
   } ;# end foreach
   append result "</table>"
   #append result "<a href='javascript:toggleAll_jb_search_pl(true)'>Select All</a> | <a href='javascript:toggleAll_jb_search_pl(false)'>Unselect All"
   return $result
}

###############################################################################
# Function   : search::searchCatalog
# Description: Search all songs (up to N) for results
#              Returns HTML shown all the tracks that matched the filter
#              Info and checkboxes are included
# Author     : Tom Wilkason
# Date       : 12/22/2002
###############################################################################
proc search::searchCatalog {parNs filter max parms} {
   variable control
   variable myns
   append result "<FORM METHOD='POST' ACTION='/$myns' NAME='jb_add_form'>"
   append result "<input type='hidden' name='jb_add_start' value='$parms'>"
   ;# Initial param will be jb_add_start, followed by each checked item as jb_add, ending with jb_append_process=Submit
   append result {<table class='songPlaylist'>}
   if {$control} {
      append result [httpHelpers::tableRow [list Chk ? P Folder "Track Name"] headerRow]
   } else {
      append result [httpHelpers::tableRow [list Chk ? Folder "Track Name"] headerRow]
   }
   ##
   # Search all files for match
   #
   set i 0
   foreach {file} [catalog::sortedList] {
      if {[string match -nocase *$filter* $file]} {
         set bgc [httpHelpers::rowStyle i]
         set track [ROOTTAIL $file]
         set folder [httpHelpers::displayFolder $file]
         if {$folder==""} {
            set flink [SP]
         } else {
            set flink [AREF "/$parNs?folder=[ENCODE [file dirname $file]]" $folder]
         }
         set cb [subst {<INPUT TYPE=checkbox NAME="jb_add" VALUE="$file">}]
         set infoBtn [httpHelpers::popupInfo $file]
         ;# The second param just performs another search using the same filter
         if {$control} {
            set playBtn [subst {<a href="/$myns?play=[ENCODE $file]&jb_search=$filter&search_what=AllFiles"  title="Play Track Immediately">$httpHelpers::playbutton</a>}]
            append result [httpHelpers::tableRow [list $cb $infoBtn $playBtn $flink $track] $bgc]
         }  else {
            append result [httpHelpers::tableRow [list $cb $infoBtn $flink $track] $bgc]
         }
      } ;# end if
      if {$i>$max} {
         break
      } ;# end if
   } ;# end foreach
   append result "</table>\n"
   if {$i>$max} {
      append result "<p>Results limited to $max matches</p>"
   } ;# end if
   append result "<INPUT TYPE=submit VALUE=Append NAME=jb_append_process>\n"
   if {$control} {
      append result "<INPUT TYPE=submit VALUE=Insert NAME=jb_insert_process>\n"
   }
   append result "</FORM>\n"
   append result [subst {<a href="javascript:toggleAll_jb_add(true)">Select All</a> | <a href="javascript:toggleAll_jb_add(false)">Unselect All</a><br>}]
   return $result
}
##
# Misc routines not in a namespace
#
proc AREF {ref txt {title ""}} {
   if {[string length $title]} {
      return "<a href='$ref' title='$title'>$txt</a>"
   } else {
      return "<a href='$ref'>$txt</a>"
   }
}
proc H2 {ref} {
   return "<h2>$ref</h2>\n"
}
proc H1 {ref} {
   return "<h1>$ref</h1>\n"
}
proc SP {args} {
   return {&nbsp;}
}
proc NOAMP {data} {
   return [string map {& &amp\;} $data]
#   return [string map {& %26} $data]
}
proc ENCODE {data} {
   return [HttpD::Encode $data]
}
proc ESCAPE {data} {
   return [string map {' \\' & %26} $data]
}
proc ROOTTAIL {file} {
   return [file root [file tail $file]]
}
proc TAILIFCAN {folder} {
   set tail [file tail $folder]
   if {[string length $tail]} {
      return $tail
   } else {
      return $folder
   }
}
##
# Http helper routines
#
namespace eval httpHelpers {
   variable appendbutton {<img border=0 src="/images/append.png">}
   variable folderclosed {<img border=0 src="/images/folderclosed.gif">}
   variable folderopen   {<img border=0 src="/images/folderopen.gif">}
   variable infobutton   {<img border=0 src="/images/info.png">}
   variable insertbutton {<img border=0 src="/images/insert.png">}
   variable playbutton   {<img border=0 src="/images/play.png">}
   variable trackimage   {<img border=0 src="/images/track.gif">}
   variable trackcropimage {<img border=0 src="/images/trackcrop.gif">}
   variable deleteimage  {<img border=0 src="/images/delete.png">}
   variable LastFile {xxxxx}
   variable InfoCache
   variable TagIDs {
      Active        Active
      "File Name"   Name
      Folder        Folder
      Artist        Artist
      Title         Title
      Album         Album
      Track         Track
      Media         Media
      Year          Year
      Genre         Genre
      Note          Note
      Comp          Comp
      Encoding      Enc
      Desc          Desc
      Size          fSize
      Date          fDate
      Duration      duration
      Position      Position
      "Sample Rate" rate
      Gain          levelFactor
   }
   array set row_styles {
      0 {lightRows}
      1 {darkRows}
   }
   ;##
   ;# Script to auto-refresh based on passed in url
   ;#
   variable refreshScript {
<script language="JavaScript">
//configure refresh interval (in seconds)
   var countDownInterval=$period;
   var countDownTime=countDownInterval+1;
   function countDown(){
      countDownTime--;
      if (countDownTime <=0) {
         countDownTime=countDownInterval;
         clearTimeout(counter);
         window.location.href = '$url'
      } else {
         // Check again in one second
         counter=setTimeout("countDown()", 1000);
      }
   }
   // Start if off
   if (document.all||document.getElementById)
      countDown();
   else
      window.onload=countDown;
</script>
   }

   proc getRefreshScript {period url} {
      variable refreshScript
      return [subst $refreshScript]
   }
   proc coverArt {folder} {
      if {[string length [coverart::covername $folder]]} {
         return [subst {<img border=0 width="$::snackAmpSettings(coversize)" height="$::snackAmpSettings(coversize)" src="/coverart?get=[ENCODE $folder]" alt="[file tail $folder]">}]
      } else {
         return ""
      }
   }
   proc rowStyle {_counter {mod 1}} {
      variable row_styles
      upvar $_counter counter
      if {[catch {incr counter} result] } then {
         set counter 0
      }
      return $row_styles([expr {($counter/$mod)&0x1}])
   }
   proc tableRow {cols {class ""}} {
      if {$class==""} {
         set result <tr>
      } else {
         set result [subst {<tr class="$class">}]
      }
      foreach {element} $cols {
         append result "<td>$element</td>"
      }
      append result "</tr>\n"
      return $result
   }
   proc inLinePlay {song} {
      return [subst {<a href="/remote/control?play=[ENCODE $song]" title="Play Track Immediately">$httpHelpers::trackimage</a>}]
   }
   proc popUpProcess {what url song} {
      switch -- $what {
         play {set im $httpHelpers::trackimage;set title "Play Track Immediately"}
         append {set im $httpHelpers::appendbutton;set title "Append Track To Playlist"}
         insert {set im $httpHelpers::insertbutton;set title "Queue Track to Play Next"}
         default {}
      }
      return [subst {<a href="javascript:textWindow('$url?$what=[ENCODE [ESCAPE $song]]')" title="$title">$im</A>}]
   }
   proc noamp {data} {
      return [string map {& %26} $data]
   }
   proc streamInfo {{class ""}} {
      set result ""
      foreach {file ip} [streamBuffer::allStreamInfo] {
         append result "[tableRow [list $file $ip] $class]\n"
      } ;# end foreach
      return $result
   }
   proc displayFolder {File} {
      global snackAmpSettings
      if {$snackAmpSettings(ListDepth) > 0} {
         ;# File join doesnt work due to single list
         set File [string map {// /} $File]      ;# remove UNC stuff
         set name [file dirname [join [lrange [file split $File] end-$snackAmpSettings(ListDepth) end] /]]
      } else {
         set name [file tail [file dirname $File]]
      }
      return $name
   }
   proc popupInfo {ref} {
      ;# Escape single quotes for javascript popup window,
      ;# Encode & to prevent confusion with parameter delimiter
      set ref "/remote/search?question=[ENCODE [ESCAPE $ref]]"
      return [subst {<a href="javascript:popupInfo('$ref',200,600,'Info')" title="Show Track Details in Pop-up">$httpHelpers::infobutton</a>}]
   }
}
###############################################################################
# Function   : trackInfo
# Description: Return, in a table, the details on some file
#
# Author     : Tom Wilkason
###############################################################################
proc httpHelpers::trackInfo {fileName {short 0}} {
   variable TagIDs
   variable InfoCache
   foreach {name tag} $TagIDs {
      set Details($tag) ""
   }
   if {[info exists InfoCache($fileName)]} {
      array set Details $InfoCache($fileName)
      after [expr {10*60000}] [list httpHelpers::clean httpHelpers::InfoCache($fileName)]
   } else {
      set tempSound [snack::sound -debug 0 -file $fileName]
      if {[catch {id3Tag::songData $fileName $tempSound Details} error] } then {
         HttpD::Log "sndInfo:$error"
         $tempSound destroy
         return $error
      }
      $tempSound destroy
      set Details(Folder) [file dirname $fileName]
      set Details(Name) [file tail $fileName]
      set Details(Position) "N/A"
      ;# Fill in non tagged data based on file name
      if {$Details(Title)==""} {
         set Details(Title) [id3Tag::guessTitle $fileName]
      }
      if {$Details(Artist)==""} {
         set Details(Artist) [id3Tag::guessArtist $fileName]
      }
      if {$Details(Album)==""} {
         set Details(Album) [id3Tag::guessAlbum $fileName]
      }
      ;# TFW:This forces table borders on blank cells, there may be a way
      ;# to do this in the table definition.
      foreach field [array names Details] {
         if {[string length $Details($field)]==0} {
            set Details($field) [SP]
         }
      } ;# end foreach
      set InfoCache($fileName) [array get Details]
   }
   if {$short} {
      set result [remote/control::songInfo Details]
   } else {
      set result [remote/control::songTable Details]
   }
}
proc httpHelpers::clean {_var} {
   upvar $_var var
   if {[info exists var]} {
      unset var
   }
}
###############################################################################
# Function   : sndInfo
# Description: Return, in a table, the details on the current song
#
# Author     : Tom Wilkason
###############################################################################
proc httpHelpers::sndInfo {{playWhere 1}} {
   Trace
   variable LastFile
   variable TagIDs
   variable Details
   global snackAmpSettings
   set fileName [soundControl::fileName]
   ;##
   ;# If the file name hasn't changed, don't
   ;# bother requerying the static song data.
   ;#
   foreach {name tag} $TagIDs {
      set Details($tag) ""
   }
   if {![string equal $fileName $LastFile]} {
      if {[catch {id3Tag::songDetails $fileName s Details} error] } then {
         HttpD::Log "sndInfo:$error"
         return
      }
      set Details(Folder) [file dirname $fileName]
      set Details(Name) [file tail $fileName]
      set LastFile $fileName
   }
   set Active [string tolower [soundControl::playState]]

   ;# Fill in non tagged data based on file name
   if {$Details(Title)==""} {
      set Details(Title) [id3Tag::guessTitle $fileName]
   }
   if {$Details(Artist)==""} {
      set Details(Artist) [id3Tag::guessArtist $fileName]
   }
   if {$Details(Album)==""} {
      set Details(Album) [id3Tag::guessAlbum $fileName]
   }
   set Details(Position) [clock format [expr {round([soundControl::position])}]  -format %M:%S]

   if {$playWhere} {
      append result [H2 "SnackAmp currently $Active on [info hostname]:$::HttpD::Httpd(port)"]
   } else {
      append result [H2 "SnackAmp currently $Active"]
   }
   ;# TFW:This forces table borders on blank cells, there may be a way
   ;# to do this in the table definition.
   foreach field [array names Details] {
      if {[string length $Details($field)]==0} {
         set Details($field) [SP]
      }
   } ;# end foreach
   append result [remote/control::songTable Details]
   append result "<br>\n"
   return $result
}
package provide urlRedirect 1.0
##
# Utility routine
#
proc dumpSocket {} {
   foreach glb [info globals Httpdsock*] {
      global $glb
      puts "Array $glb--------------"
      catch {parray $glb}
   } ;# end foreach
}

set httpToDo {
- Playlists on play_control
- Ability to change the queued print out size
- FF/FR to play control
- Figure a way to push data to browser without it requesting it
  Have knowlege when a socket is open.
- Clicking on a song should bring up a pop-up with song details, use
  play button to play a song (can use javascript like my pop-up)
- Animated gif to denote if a song is playing

Unix bugs
- KDE/Netscape the meta refresh doesn't seem to work
- Pausing a song doesn't get an indication on the browser,
  it only knows if it is stopped or playing.
}

